// Import necessary modules from d3 library
import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot'; // Import custom scatterPlot module

// URL to the CSV data
const csvUrl =
  'https://gist.githubusercontent.com/sagam929/c391612d2be3ad2d5b6891c2c3221972/raw/2acf19275a2eb7c162617a40cf938b3f3b68a2f9/india.csv';

// Function to parse each row of the CSV data
const parseRow = (d) => {
  // Convert certain columns to numbers
  d['Avg Ratio of Boys and Girls'] =
    +d['Avg Ratio of Boys and Girls'];
  d.Library = +d.Library;
  d['Play Ground'] = +d['Play Ground'];
  d['Computer Labs'] = +d['Computer Labs'];
  d['Food Stalls'] = +d['Food Stalls'];
  return d;
};

// Set the desired width and height for the SVG container
const width = 900;
const height = 500;

// Create an SVG element and append it to the body of the HTML document
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

// Add x-axis label to the SVG
const xAxisLabelElement = svg
  .append('text')
  .attr('x', width / 2)
  .attr('y', height + 1)
  .style('text-anchor', 'middle')
  .style('font-weight', 'bold')
  .text('Avg Ratio of Boys and Girls Label');

// Add y-axis label to the SVG
const yAxisLabelElement = svg
  .append('text')
  .attr('transform', 'rotate(-90)')
  .attr('x', -height / 2)
  .attr('y', 19)
  .style('text-anchor', 'middle')
  .style('font-weight', 'bold')
  .text('Library'); // Initial y-axis label

// Array containing all possible y-axis labels
let yAxisLabels = [
  'Library',
  'Play Ground',
  'Computer Labs',
  'Food Stalls',
];
let yAxisIndex = 0; // Index to keep track of the current y-axis label

// Asynchronous main function to load data and render the scatter plot
const main = async () => {
  // Load CSV data and parse each row using the provided function
  const data = await csv(csvUrl, parseRow);

  // Initialize scatter plot with initial configuration
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(data)
    .xValue((d) => d['Avg Ratio of Boys and Girls']) // Static x-axis label
    .yValue((d) => d[yAxisLabels[yAxisIndex]]) // Dynamic y-axis label
    .margin({
      top: 50, // Increased top margin to accommodate axis labels
      right: 20,
      bottom: 40,
      left: 50,
    })
    .radius(5);

  // Render the initial scatter plot
  svg.call(plot);

  // Set interval to update the scatter plot periodically
  setInterval(() => {
    // Cycle through y-axis labels
    yAxisIndex = (yAxisIndex + 1) % yAxisLabels.length;
    const newYAxisLabel = yAxisLabels[yAxisIndex];

    // Update y-axis value accessor
    plot.yValue((d) => d[newYAxisLabel]);

    // Update y-axis label text
    yAxisLabelElement.text(`${newYAxisLabel} Label`);

    // Remove previous plot
    svg.selectAll('.plot-group').remove();

    // Render updated scatter plot
    svg.call(plot);
  }, 2000); // Interval duration: 2000 milliseconds (2 seconds)
};

// Call the main function to start the process
main();
