// Importing required modules
import {
  scaleLinear,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
  symbols,
  symbol,
  schemeCategory10,
} from 'd3';

// Define scatterPlot function
export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let xLabel;
  let yLabel;
  let margin;
  let radius;

  // Define the main logic for generating the scatter plot
  const my = (selection) => {
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      category: d.State_UT, // Using State_UT as the categorical variable
    }));

    var uniqueStates = Array.from(
      new Set(data.map((d) => d.State_UT)),
    );

    var shape = scaleOrdinal(
      uniqueStates,
      symbols.map((s) => symbol().type(s)()),
    );
    var color = scaleOrdinal(
      uniqueStates,
      schemeCategory10,
    );

    const circles = selection
      .selectAll('.dots')
      .data(marks)
      .join(
        (enter) =>
          enter.append('path').attr('class', 'dots'),
        (update) => update,
        (exit) => exit.remove(),
      )
      .attr('d', (d) => shape(d.category))
      .attr('fill', (d) => color(d.category))
      .attr('transform', (d) => `translate(${d.x},${d.y})`)
      .attr('r', radius); // Set the radius of the circles

    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y))
      .append('text')
      .attr('class', 'axis-label')
      .attr('y', -margin.left + 10)
      .attr('x', -height / 2)
      .attr('fill', 'black')
      .attr('transform', 'rotate(-90)')
      .attr('text-anchor', 'middle')
      .text(yLabel); // Set y-axis label

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`,
      )
      .call(axisBottom(x))
      .append('text')
      .attr('class', 'axis-label')
      .attr('y', margin.bottom - 10)
      .attr('x', width / 2)
      .attr('fill', 'black')
      .attr('text-anchor', 'middle')
      .text(xLabel); // Set x-axis label

    // Add x-axis label
    selection
      .append('text')
      .attr('class', 'x-axis-label')
      .attr('x', width / 2)
      .attr('y', height - margin.bottom / 2)
      .attr('text-anchor', 'middle')
      .attr('fill', 'black')
      .text(xLabel);

    // Add y-axis label
    selection
      .append('text')
      .attr('class', 'y-axis-label')
      .attr('x', -height / 2)
      .attr('y', margin.left / 2)
      .attr('text-anchor', 'middle')
      .attr('transform', 'rotate(-90)')
      .attr('fill', 'black')
      .text(yLabel);
  };

  // Accessors for setting parameters
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : 300;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : 300;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.xLabel = function (_) {
    return arguments.length ? ((xLabel = _), my) : xLabel;
  };

  my.yLabel = function (_) {
    return arguments.length ? ((yLabel = _), my) : yLabel;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
